import fuzz from "fuzzball";  // todo: implement for fuzzy match
import { Filter, MatchingAlgorithms } from "../../generated/prisma";

function splitMatch(filterValue: string): string[] {
    const findTerms = [...filterValue.matchAll(/"([^"]+)"|(\S+)/g)];
    return findTerms.map(value => {
        const term = value[1] || value[2];
        return term.trim().replace(/\s+/g, "\\s+").replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
    });
}

export const all = (filter: Filter, input: string) => {
    throw new Error("'all' filter not implemented");
};

export const any = (filter: Filter, input: string) => {
    // const words = splitMatch(filter.value);
    // console.log(JSON.stringify(words));
    // return words.some(word => new RegExp(`\\b${word}\\b`).test(input));

    // TODO:
    //      The below code works, but I'd like to use regex as was done in pyrss.
    //      The above commented code does not work, figure it out.


    const inputWords = input.split(" ").map(word => filter.is_insensitive ? word.toLowerCase() : word);
    const filterWords = filter.value.split(" ").map(word => filter.is_insensitive ? word.toLowerCase() : word);

    for (const filterWord of filterWords) {
        if (inputWords.includes(filterWord)) {
            console.log(input);
            return true;
        }
    }

    return false;
};

export const literal = (filter: Filter, input: string) => {
    throw new Error("'literal' filter not implemented");
};

export const regex = (filter: Filter, input: string) => {
    try {
        const flags = filter.is_insensitive ? "i" : "";
        return new RegExp(filter.value, flags).test(input);
    } catch (error) {
        console.error(`Invalid regex pattern: ${filter.value}`, error);
        return false;
    }
};

export const fuzzy = (filter: Filter, input: string) => {
    throw new Error("'fuzzy' filter not implemented");
};

export const mapAlgorithmToFunction = (filter: Filter, input: string) => {
    switch (filter.matching_algorithm) {
        case MatchingAlgorithms.ALL: return all(filter, input);
        case MatchingAlgorithms.ANY: return any(filter, input);
        case MatchingAlgorithms.EXACT: return literal(filter, input);
        case MatchingAlgorithms.REGEX: return regex(filter, input);
        case MatchingAlgorithms.FUZZY: return fuzzy(filter, input);
        default:
            throw new Error(`Unknown algorithm: ${filter.matching_algorithm}`);
    }
};